package com.iqball.app.serialization

import arrow.core.Either
import com.squareup.moshi.JsonAdapter
import com.squareup.moshi.JsonReader
import com.squareup.moshi.JsonWriter
import com.squareup.moshi.Moshi
import com.squareup.moshi.rawType
import java.lang.ClassCastException
import java.lang.reflect.ParameterizedType
import java.lang.reflect.Type


private class EitherTypeAdapter(val leftType: JsonPrimitiveType, val rightType: JsonPrimitiveType) :
    JsonAdapter<Either<*, *>>() {
    override fun fromJson(reader: JsonReader): Either<*, *>? {
        val value = reader.readJsonValue() ?: return null

        val valueJsonType = value.javaClass.getJsonPrimitive()
        if (valueJsonType == leftType)
            return Either.Left(value)

        if (valueJsonType == rightType)
            return Either.Right(value)
        throw ClassCastException("Cannot cast a value of type " + value.javaClass + " as either " + leftType.name.lowercase() + " or " + rightType.name.lowercase())
    }


    override fun toJson(writer: JsonWriter, value: Either<*, *>?) {
        when (value) {
            is Either.Left -> writer.jsonValue(value.value)
            is Either.Right -> writer.jsonValue(value.value)
            null -> writer.nullValue()
        }
    }


}

object EitherTypeAdapterFactory: JsonAdapter.Factory {
    override fun create(
        type: Type,
        annotations: MutableSet<out Annotation>,
        moshi: Moshi
    ): JsonAdapter<*>? {
        if (type !is ParameterizedType)
            return null
        if (type.rawType != Either::class.java)
            return null
        val leftType = type.actualTypeArguments[0].rawType.getJsonPrimitive()
        val rightType = type.actualTypeArguments[1].rawType.getJsonPrimitive()
        if (leftType == rightType) {
            throw UnsupportedOperationException("Cannot handle Either types with both sides being object, array, string or number. Provided type is : $type")
        }
        return EitherTypeAdapter(leftType, rightType)
    }
}

private enum class JsonPrimitiveType {
    Array,
    Object,
    String,
    Number;
}

private fun Class<*>.getJsonPrimitive(): JsonPrimitiveType {
    if (isPrimitive)
        return JsonPrimitiveType.Number
    if (isArray)
        return JsonPrimitiveType.Array
    if (this == String::class.java)
        return JsonPrimitiveType.String
    return JsonPrimitiveType.Object
}